%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Solve Navier-Stokes equation using the finite element method.       %
% Equations U.grad(U) = -grad(P) + 1/Re grad.(grad(U)) and grad.U = 0 %
% are solved with measured PC-MRI constraints.                        %
%                                                                     %
% Reference: Gresho PM et al. Incompressible Flow and the Finite      %
% Element Method, 1998.                                               % 
%                                                                     %
% This code is related to the paper "Finite-Element Computational     % 
% Fluid Dynamics Simulations Constrained by Phase-Contrast MRI Data"  %
% submitted to the Proceedings of the 24th Scientific Meeting,        %
% International Society for Magnetic Resonance in Medicine,           %
% Singapore, 2016.                                                    %
%                                                                     %
% Authors: Borges, G. B. F., Siqueira, I. R., Carvalho, J. L. A.,     %
% Nielsen, J. F., Rispoli, V. C.                                      %
%                                                                     % 
% Corresponding author: Prof. Vinicius Rispoli (project advisor)      %
% Unb Gama College - University of Brasilia                           %
% Brasilia - Brazil                                                   %
% E-mail: vrispoli@pgea.unb.br / vrispoli@unb.br                      %
%                                                                     %
% If you intend to use this code, please cite our paper :)            %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clear all, clc, close all;

%load MRI data
load Wmri 
load Umri
load Mag

%define mesh boundary
[malhaVoxels, nVoxelsX, nVoxelsY] = limpaRuido(mag);

[coordGlobal, conectMatrix, malhaNodesFixed, contornoQuad, coordGlobalQuad, conectMatrixQuad, malhaNodesQuad, malhaVoxelsQuad] = geraMalha(malhaVoxels, nVoxelsX, nVoxelsY);

[NE, NI, NS, NDE, NDI, contornoNodes] = geraContorno(malhaNodesQuad, contornoQuad, 2*nVoxelsX, 2*nVoxelsY);

V = 5e-1;%V = 0.5m/s
L = 1e-3;%L = 1mm
MU = 5e-3;%mu = 5mPa.s 
RHO = 1100;%rho = 1100Kg/m^3

Re = RHO*V*L/MU;%Re = 110

ro  = 1;

mi  = 1/Re;
lambda = 1e-2;

umri = -umri./pi;
wmri = wmri./pi;

DomNodeID = conectMatrixQuad';
X = coordGlobalQuad(:, 1)';
Y = coordGlobalQuad(:, 2)';
NELE = size(conectMatrixQuad, 1);
NODES = size(coordGlobalQuad, 1);

[velocidadeEntrada] = velocidadeContornoEnt(umri, wmri);

% Relacao da numeracao local e global dos graus de liberdade
[NDoF, ASSMtrx] = dofdrive(NELE, NODES, DomNodeID);

[C] = Newton(wmri, umri, velocidadeEntrada, NE, NI, NS, NDE, NDI, NELE, NDoF, ASSMtrx, DomNodeID, X, Y, ro, mi, lambda);
plotaCamposUWP(malhaNodesQuad, 2*nVoxelsX, 2*nVoxelsY, NELE, NODES, DomNodeID, ASSMtrx, X, Y, C);

plotaMalhaQuad(NODES,  coordGlobalQuad, conectMatrixQuad)
plotaMalha(772,  coordGlobal, conectMatrix)


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
